--[[
-- Author: Garth Gibson (titanpvplog@shaw.ca)
--]]

local enteredWorld = false;
local variablesLoaded = false;
local realm = "";
local foundName = false;
local toonName = "";
local myList = { };
local playersThatNoLongerExist = { };
local frndName = "";
local slashmsg = "";
local GREEN   = "|cff6bb700";
local RED     = "|cffbe0303";
local WHITE   = "|cffdedede";


function RealmFriends_OnLoad()
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("CHAT_MSG_SYSTEM");
	SLASH_REALMFRIENDS1 = "/realmfriends";
	SLASH_REALMFRIENDS2 = "/rfs";
	SlashCmdList["REALMFRIENDS"] = RealmFriends_SlashHandler;
end

function RealmFriends_OnEvent()
	if(event == "PLAYER_ENTERING_WORLD") then
		enteredWorld = true;
		if(variablesLoaded) then
			RealmFriends_Initialize();
			RealmFriends_isThisFirstRun();
		end
	elseif(event == "VARIABLES_LOADED") then
		variablesLoaded = true;
		if(enteredWorld) then
			RealmFriends_Initialize();
			RealmFriends_isThisFirstRun();
		end
	elseif(event == "CHAT_MSG_SYSTEM") then
		MarsMessageParser_ParseMessage("RealmFriends", arg1);
	end
end

function RealmFriends_Initialize()
	-- register MarsMessageParser functions
	MarsMessageParser_RegisterFunction("RealmFriends", ERR_FRIEND_ADDED_S, RealmFriends_SaveFriend, false);
	MarsMessageParser_RegisterFunction("RealmFriends", ERR_FRIEND_REMOVED_S, RealmFriends_DeleteFriend, false);
	MarsMessageParser_RegisterFunction("RealmFriends", ERR_IGNORE_ADDED_S, RealmFriends_SaveIgnore, false);
	MarsMessageParser_RegisterFunction("RealmFriends", ERR_IGNORE_REMOVED_S, RealmFriends_DeleteIgnore, false);

	-- initialize variable defaults
	realm = GetCVar("realmName");
	toonName = UnitName("player");
	if(syncFriends == NIL) then
		syncFriends = false;
	end
	if(syncIgnores == NIL) then
		syncIgnores = false;
	end
	if(syncAllFriendsEnabled == NIL) then
		syncAllFriendsEnabled = false;
	elseif(syncAllFriendsEnabled) then
		syncFriends = true;
	end
	if(syncAllFriendsDisabled == NIL) then
		syncAllFriendsDisabled = false;
	elseif(syncAllFriendsDisabled) then
		syncFriends = false;
	end
	if(syncAllIgnoresEnabled == NIL) then
		syncAllIgnoresEnabled = false;
	elseif(syncAllIgnoresEnabled) then
		syncIgnores = true;
	end
	if(syncAllIgnoresDisabled == NIL) then
		syncAllIgnoresDisabled = false;
	elseif(syncAllIgnoresDisabled) then
		syncIgnores = false;
	end
	if(friendsToKeep == NIL) then
		friendsToKeep = { };
	end
	if(friendsToKeep[realm] == NIL) then
		friendsToKeep[realm] = { };
	end
	if(friendsToDelete == NIL) then
		friendsToDelete = { };
	end
	if(friendsToDelete[realm] == NIL) then
		friendsToDelete[realm] = { };
	end
	if(ignoresToKeep == NIL) then
		ignoresToKeep = { };
	end
	if(ignoresToKeep[realm] == NIL) then
		ignoresToKeep[realm] = { };
	end
	if(ignoresToDelete == NIL) then
		ignoresToDelete = { };
	end
	if(ignoresToDelete[realm] == NIL) then
		ignoresToDelete[realm] = { };
	end
	if(myNames == NIL) then
		myNames = { };
	end
	if(myNames[realm] == NIL) then
		myNames[realm] = { };
	end
	RealmFriends_ChatMsg(LOADMSG);
end

function RealmFriends_isThisFirstRun()	
	-- If this is not the first time running Realm Friends sync lists
	if(RealmFriends_isNameOnList(toonName, myNames[realm])) then
		if(syncFriends) then
			RealmFriends_syncFriends();
		end
		if(syncIgnores) then
			RealmFriends_syncIgnores();
		end
	-- If this is the first time running Realm Friends show the configuration window
	else
		tinsert(myNames[realm], toonName);
		if(not syncAllFriendsEnabled) and (not syncAllIgnoresEnabled) and (not syncAllFriendsDisabled) and (not syncAllIgnoresDisabled) then
				ShowUIPanel(RealmFriendsUIFrame);
		end
	end
end

function RealmFriends_syncFriends()
	-- delete players from the friends list FIRST so when we add players from the friends list into the save file, we won't add these players.
	table.foreach(friendsToDelete[realm], function(idx, name)
	RealmFriends_getFriends();
		if(RealmFriends_isNameOnList(name, myList)) then
			RemoveFriend(name);
		end
	end);

	-- add players to the friends list
	table.foreach(friendsToKeep[realm], function(idx, name)
		-- [[
		-- players may keep their alt/main names on their friends list
		-- avoid trying to add your own name to your friends list
		-- ]]
		RealmFriends_getFriends();
		if(name ~= toonName) then
			if(not RealmFriends_isNameOnList(name, myList)) then			
				AddFriend(name);
			end
		end
	end);

	-- add friends to the saved list
	RealmFriends_getFriends();
	table.foreach(myList, function(idx, name)
		-- add friend to saved list if it's not there already
		RealmFriends_SaveFriend(name);
	end);
	
	-- delete saved friends that no longer exist
	-- playersThatNoLongerExist = { };
	-- table.foreach(friendsToKeep[realm], function(idx, name)
	-- RealmFriends_getFriends();
	-- 	if(name ~= toonName) then
	--		if(not RealmFriends_isNameOnList(name, myList)) then			
	--	 		table.insert(playersThatNoLongerExist, name);
	--	 	end
	--	end
	-- end);
	-- table.foreach(playersThatNoLongerExist, function(idx, name)
	-- 	RealmFriends_DeleteFriend(name);
 	-- end);
end

function RealmFriends_syncIgnores()
	-- delete players from ignore list FIRST so when we add players from the ignore list into the save file, we won't add these players.
	table.foreach(ignoresToDelete[realm], function(idx, name)
		RealmFriends_getIgnores();
		if(RealmFriends_isNameOnList(name, myList)) then
			DelIgnore(name);
		end
	end);
	
	-- add players to the ignore list
	table.foreach(ignoresToKeep[realm], function(idx, name)
		-- avoid trying to add your own name to your ignores list
		RealmFriends_getIgnores();
		if(name ~= toonName) then
			if(not RealmFriends_isNameOnList(name, myList)) then
				AddIgnore(name);
			end
		end
	end);

	-- add ignored players to the saved list
	RealmFriends_getIgnores();
	table.foreach(myList, function(idx, name)
		RealmFriends_SaveIgnore(name);
	end);
end

function RealmFriends_SaveFriend(name)
	if(syncFriends) then
		-- add to friends to keep list.
		if(not RealmFriends_isNameOnList(name, friendsToKeep[realm])) then
			tinsert(friendsToKeep[realm], name);
		end
		-- remove from deleted friends list
		RealmFriends_removeFromList(name, friendsToDelete[realm]);
	end
end

function RealmFriends_DeleteFriend(name)
	if(syncFriends) then
		-- add to deleted friends list.
		if(not RealmFriends_isNameOnList(name, friendsToDelete[realm])) then
			tinsert(friendsToDelete[realm], name);
		end
		-- remove from friends to keep list.
		RealmFriends_removeFromList(name, friendsToKeep[realm]);
	end
end

function RealmFriends_SaveIgnore(name)
	if(syncIgnores) then
		-- add to ignores to keep list
		if(not RealmFriends_isNameOnList(name, ignoresToKeep[realm])) then
			tinsert(ignoresToKeep[realm], name);
		end			
		-- remove from ignores to delete list
		RealmFriends_removeFromList(name, ignoresToDelete[realm]);
	end
end

function RealmFriends_DeleteIgnore(name)
	if(syncIgnores) then
		-- add to ignores to delete list.
		if(not RealmFriends_isNameOnList(name, ignoresToDelete[realm])) then
			tinsert(ignoresToDelete[realm], name);
		end	
		-- remove from ignores to keep list.
		RealmFriends_removeFromList(name, ignoresToKeep[realm]);
	end
end

function RealmFriends_isNameOnList(name, list)
	foundName = false;
	table.foreach(list, function(savedIdx, savedName)
		if(savedName == name) then
			foundName = true;
		end
	end);
	return foundName;
end

function RealmFriends_removeFromList(name, list)
	table.foreach(list, function(listIdx, listName)
		if(listName == name) then
			table.remove(list, listIdx);
		end
	end);
end

function RealmFriends_getFriends()
	myList = { };
	for idx = 1, GetNumFriends() do
		frndName, level, class, area, connected, status  = GetFriendInfo(idx);
		tinsert(myList, frndName);
	end
	return myList;
end

function RealmFriends_getIgnores()
	myList = { };
	for idx = 1, GetNumIgnores() do
		frndName = GetIgnoreName(idx);
		tinsert(myList, frndName);
	end
	return myList;
end

function RealmFriends_enableAllFriends()
	syncAllFriendsEnabled = true;
	syncFriends = true;
	syncAllFriendsDisabled = false;
	RealmFriends_ChatMsg(SYNCALLFRIENDS .. " " .. GREEN .. ON);
	RealmFriends_syncFriends();
end

function RealmFriends_disableAllFriends()
	syncAllFriendsEnabled = false;
	syncFriends = false;
	syncAllFriendsDisabled = true;
	RealmFriends_ChatMsg(SYNCALLFRIENDS .. " " .. RED .. OFF);
end

function RealmFriends_toggleMyFriends(val)
	syncAllFriendsEnabled = false;
	syncFriends = val;
	syncAllFriendsDisabled = false;
	if(syncFriends) then
		RealmFriends_ChatMsg(SYNCMYFRIENDS .. " " .. GREEN .. ON);
		RealmFriends_syncFriends();
	else
		RealmFriends_ChatMsg(SYNCMYFRIENDS .. " " .. RED .. OFF);
	end
end

function RealmFriends_enableAllIgnores()
	syncAllIgnoresEnabled = true;
	syncIgnores = true;
	syncAllIgnoresDisabled = false;
	RealmFriends_ChatMsg(SYNCALLIGNORES .. " " .. GREEN .. ON);
	RealmFriends_syncIgnores();
end

function RealmFriends_disableAllIgnores()
	syncAllIgnoresEnabled = false;
	syncIgnores = false;
	syncAllIgnoresDisabled = true;
	RealmFriends_ChatMsg(SYNCALLIGNORES .. " " .. RED .. OFF);
end

function RealmFriends_toggleMyIgnores(val)
	syncAllIgnoresEnabled = false;
	syncIgnores = val;
	syncAllIgnoresDisabled = false;
	if(syncIgnores) then
		RealmFriends_ChatMsg(SYNCMYIGNORES .. " " .. GREEN .. ON);
		RealmFriends_syncIgnores();
	else
		RealmFriends_ChatMsg(SYNCMYFRIENDS .. " " .. RED .. OFF);
	end
end

function RealmFriends_SlashHandler(command)
	if(command ~= NIL) then
		if(string.lower(command) == string.lower(FRIENDS .. " " .. ON)) then
			RealmFriends_toggleFriends(true);
		elseif(string.lower(command) == string.lower(FRIENDS .. " " .. OFF)) then
			RealmFriends_toggleFriends(false);	
		elseif(string.lower(command) == string.lower(IGNORE .. " " .. ON)) then
			RealmFriends_toggleIgnore(true);
		elseif(string.lower(command) == string.lower(FRIENDS .. " " .. OFF)) then
			RealmFriends_toggleIgnore(false);
		elseif(string.lower(command) == string.lower(FRIENDS .. " " .. ALL)) then
			RealmFriends_enableAllFriends();
		elseif(string.lower(command) == string.lower(FRIENDS .. " " .. NONE)) then
			RealmFriends_disableAllFriends();
		elseif(string.lower(command) == string.lower(IGNORE .. " " .. ALL)) then
			RealmFriends_enableAllIgnores();
		elseif(string.lower(command) == string.lower(IGNORE .. " " .. NONE)) then
			RealmFriends_disableAllIgnores();		
		elseif(string.lower(command) == string.lower(CONFIG)) then
			ShowUIPanel(RealmFriendsUIFrame);
		else
			RealmFriends_ShowSlashUsage();
		end
	else
		RealmFriends_ShowSlashUsage();
	end
end

function RealmFriends_ShowSlashUsage()
	slashmsg = GREEN .. REALMFRIENDSTITLE .. "\n</rfs " .. FRIENDS .. " ";
	if(syncFriends) then
		slashmsg = slashmsg .. WHITE .. ON .. GREEN .. " | " .. OFF .. ">";
	else
		slashmsg = slashmsg .. ON .. " | " .. WHITE .. OFF .. ">";
	end
	slashmsg = slashmsg .. GREEN .. "\n</rfs " .. IGNORE .. " ";
	if(syncIgnores) then
		slashmsg = slashmsg .. WHITE .. ON .. GREEN .. " | " .. OFF .. ">";
	else
		slashmsg = slashmsg .. ON .. " | " .. WHITE .. OFF .. ">";
	end
	slashmsg = slashmsg .. GREEN .. "\n</rfs " .. CONFIG .. ">";
	RealmFriends_ChatMsg(slashmsg)
end

function RealmFriends_ChatMsg(msgText)
	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage(msgText);
	end
end
